/*
 * Simulator of microcontrollers (mem.cc)
 *
 * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of microcontroller simulator: ucsim.

UCSIM is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UCSIM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UCSIM; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include <stdio.h>
#include <stdlib.h>

#include "utils.h"
#include "globals.h"

#include "memcl.h"
#include "hwcl.h"


/*
 * Memory location handled specially by a hw element
 */

cl_memloc::cl_memloc(long addr):
  cl_base()
{
  address= addr;
  hws= new cl_list(2, 2);
  hws->init();
}

cl_memloc::~cl_memloc(void)
{
  hws->disconn_all();
  delete hws;
}

ulong
cl_memloc::read(class cl_mem *mem)
{
  uchar ret= 0;
  class cl_hw *hw;

  if (!hws ||
      hws->count == 0)
    return(ret);
  if (hw= (class cl_hw *)(hws->at(0)))
    ret= hw->read(mem, address);
  return(ret);
}

void
cl_memloc::write(class cl_mem *mem, long addr, ulong *val)
{
  class cl_hw *hw;
  int i;

  if (!hws)
    return;
  for (i= 0; i < hws->count; i++)
    {
      hw= (class cl_hw *)hws->at(0);
      hw->write(mem, addr, val);
    }
}


/* Sorted collection of memory locations */

cl_memloc_coll::cl_memloc_coll(void):
  cl_sorted_list(2, 2)
{
  Duplicates= FALSE;
}

void *
cl_memloc_coll::key_of(void *item)
{
  return(&(((class cl_memloc *)item)->address));
}

int
cl_memloc_coll::compare(void *key1, void *key2)
{
  if (*(long*)key1 > *(long*)key2)
    return(1);
  else
    if (*(long*)key1 < *(long*)key2)
      return(-1);
    else
      return(0);
}

class cl_memloc *
cl_memloc_coll::get_loc(long address)
{
  t_index i;

  if (search(&address, i))
    return((class cl_memloc*)(at(i)));
  return(0);
}


/*
 * Memory
 */

cl_mem::cl_mem(enum mem_class atype, long asize):
  cl_base()
{
  type= atype;
  mem = (uchar*)malloc(size= asize);
  read_locs= new cl_memloc_coll();
  write_locs= new cl_memloc_coll();
}

cl_mem::~cl_mem(void)
{
  if (mem)
    free(mem);
  delete read_locs, write_locs;
}

char *
cl_mem::id_string(void)
{
  char *s= get_id_string(mem_ids, type);

  return(s?s:"NONE");
}

ulong
cl_mem::read(long addr)
{
  class cl_memloc *loc;

  if (addr >= size)
    return(0);
  if (loc= read_locs->get_loc(addr))
    return(loc->read(this));
  return(mem[addr]);
}

ulong
cl_mem::get(long addr)
{
  return(addr<size?mem[addr]:0);
}

void
cl_mem::write(long addr, ulong *val)
{
  class cl_memloc *loc;

  if (addr >= size)
    return;
  if (loc= write_locs->get_loc(addr))
    loc->write(this, addr, val);
  mem[addr]= *val;
}

void
cl_mem::set(long addr, ulong val)
{
  if (addr < size)
    mem[addr]= val;
}


/* End of mem.cc */
